#!/bin/sh
#/****************************************************************************/
#/* SNMP SETUP SCRIPT                                                        */
#/*                                      Copyright FUJITSU LIMITED 2013      */
#/*                                                                          */
#/* $Name: SLINSVMCO-2_3_0-0 $                                                                */
#/****************************************************************************/
# Development date : Wed Nov 20 2013
# For Cassiopeia
#==============================================================================
# This is the script to add or remove the snmp setting for SVMCO.
# Only SVmco Installer is permitted to execute this script.
# This script will return as follows.
#      0 : End successfully
#      1 : Abnormal end
#==============================================================================

SNMPTRPDFILE=/etc/snmp/snmptrapd.conf
SNMPTRAPRC=/etc/init.d/snmptrapd

TRAPCONFTMP=/tmp/snmptrapd.conf

STARTLINE="## Setup for SVmco Please do not remove the following lines ##"
STOPLINE="## End of Setup for SVmco ##"

SNMPTRAPRCNEW=/etc/init.d/snmptrapd.rpmnew
SNMPTRAPRCBK=/var/tmp/snmptrapd.org
SNMPTRAPOPT=""

SNMPTRAPFILE=""
SNMPTRAPFILE1=snmptrapd
SNMPTRAPFILE2=snmptrapd.options

SNMPTRAPOPTBK=""
SNMPTRAPOPTBK1=/var/tmp/snmptrapd.org
SNMPTRAPOPTBK2=/var/tmp/snmptrapd.options.org

TRAPDOID=".1.3.6.1.4.1.3183.1.1.0.12611328 \
		  .1.3.6.1.4.1.3183.1.1.0.12611329 \
		  .1.3.6.1.4.1.3183.1.1.0.12611330 \
		  .1.3.6.1.4.1.3183.1.1.0.12611331 \
		  .1.3.6.1.4.1.3183.1.1.0.12611332 \
		  .1.3.6.1.4.1.3183.1.1.0.12611333 \
		  .1.3.6.1.4.1.3183.1.1.0.12611334 \
		  .1.3.6.1.4.1.3183.1.1.0.12611335 \
		  .1.3.6.1.4.1.3183.1.1.0.12611336 \
		  .1.3.6.1.4.1.3183.1.1.0.12611337 \
 		  .1.3.6.1.4.1.3183.1.1.0.12611338"
TRAPCMD=/opt/fujitsu/SVmco/bin/pcxtrp
TRAPOPT=/
TRAPCOMMUNITY=public


#-----------------------------------------------------------------------------
# snmptrapd options conversion function
#
#  Brief;
#  snmptrapd options setting for SVMCO (-Lsd <--> -n -Lf /dev/null)
#  If target word is not found, options setting is ignored.
#
#  Args;
#  $1       :target file path
#  $2       :target word
#  $3       :changes (for sed command)
#  Return;
#  0        :Normal end
#-----------------------------------------------------------------------------
function trapd_opt_conv()
{
	filepath=$1
	targetwd=$2
	change=$3

	ncnt=`cat $filepath | wc -l`
	zcnt=`grep $ $filepath | wc -l`
	
	if [ $ncnt -ne $zcnt ]
	then
	    echo "" >> $filepath
	fi
	
	touch $filepath.org
	chmod --reference $filepath $filepath.org
	
        bkup=$IFS
        IFS=
        
        exec 3< $filepath
        
        while read target 0<&3
        do
	    # Check whether OPTION line or not.(ignore comment line)
	    targetchk=`echo $target | egrep "$targetwd" | wc -w`
	    
	    # In case of targetwd line, change .
	    if [ $targetchk -ne 0 ]
	    then
	        temp=`echo  "$target" | sed "$change"`
	        echo "$temp" >> $filepath.org
	    else
	        echo "$target" >> $filepath.org
	    fi
	    
	done
	
	mv $filepath.org $filepath
	
        exec 3<&-
        
        IFS=$bkup
	
	return 0
}


#-----------------------------------------------------------------------------
# main
#
#  Brief;
#  snmp setup function
#
#  Args;
#
#  Return;
#  0        :Normal end
#  1        :Abnormal end
#-----------------------------------------------------------------------------

type=$1

#---------------------------------- auth check ------------------------------------
WHO=`whoami | egrep "^root$" | wc -l`
if [ $WHO -ne 1 ]
	then
		echo "snmpsetup.sh : Operation not permitted. root only"
		exit 1
fi

level1=`/sbin/runlevel | grep -P 'unknown|..S' | wc -l`

#---------------------------------- OS check ------------------------------------
which systemctl > /dev/null 2>&1
result=$?

if [ $? -ne 1 ]
    then
#RHEL7 or later
    OS_TYPE=1
else
    OS_TYPE=0
fi


#--------------------------------- snmptrapd stop -------------------------------------

case $type in
	'install'|'upinstall'|'uninstall')

	if [ $OS_TYPE -eq 0 ]
	then
	    /sbin/service snmptrapd stop > /dev/null 2>&1
	else
	    /bin/systemctl stop snmptrapd > /dev/null 2>&1
	fi
	;;
esac

#------------------------------ snmptrapd options setting start ----------------------

if [ $OS_TYPE -ne 1 ]
then
if ! [ -f $SNMPTRAPRC -a -w $SNMPTRAPRC ]
then
	echo "Can't find the file : \"$SNMPTRAPRC\""
	exit 1
fi

if [ -f $SNMPTRAPRC.org ]
then
	rm -f $SNMPTRAPRC.org
fi

case $type in
	'upinstall')
		#
		# init script backup
		#
		cp $SNMPTRAPRC $SNMPTRAPRCBK
		
		#
		# option setting check
		#
		chg=0
		n1=`egrep -c "^[[:space:]]*OPTIONS" $SNMPTRAPRC`
		n2=`egrep -c "^[[:space:]]*OPTIONS[[:space:]]*=[[:space:]]*\"[[:space:]]*-n -Lf \/dev\/null -p \/var\/run\/snmptrapd.pid\"$" $SNMPTRAPRC`
		if [ $n1 -ne $n2 ]; then
			chg=1
		fi
		
		#
		# snmptrapd init script repair
		#
		if [ $chg -eq 0 -a -f $SNMPTRAPRCNEW ];then
		    mv $SNMPTRAPRCNEW $SNMPTRAPRC
		else
		    n3=`egrep -c "([[:space:]]*|\")-n -Lf \/dev\/null([[:space:]]*|\")" $SNMPTRAPRC`
		    if [ $n3 -ne 0 ]; then
		        trapd_opt_conv $SNMPTRAPRC "^[[:space:]]*OPTIONS" "s/-n -Lf \/dev\/null/-Lsd/"
		    fi
		fi

		;;
esac
fi

#
# snmptrapd.options file path check
#
l1=0

# ** 2010.8.24 For RHEL6
# (introduce /etc/sysconfig/snmptrapd. Use it to specify snmptrapd command
#  line options.  /etc/sysconfig/snmptrapd.options is not used anymore. v.5.5-2) ***

NETSNMPVL=`rpm -qa net-snmp | grep "net-snmp-5\.[0-4]" | wc -l`
if [ $NETSNMPVL -eq 0 ]
then
    SNMPTRAPFILE=$SNMPTRAPFILE1
    SNMPTRAPOPTBK=$SNMPTRAPOPTBK1
else
    SNMPTRAPFILE=$SNMPTRAPFILE2
    SNMPTRAPOPTBK=$SNMPTRAPOPTBK2
fi
# 2010.8.24 **************************************************

if [ $OS_TYPE -ne 1 ] ; then
while read target ;do
    targetchk1=`echo $target | egrep "^[[:space:]]*\.[[:space:]]\/([[:alnum:]]|\/)*\/$SNMPTRAPFILE" | wc -l`
    targetchk2=`echo $target | egrep "^[[:space:]]*fi$" | wc -l`
    targetchk3=`echo $target | egrep "^[[:space:]]*OPTIONS[[:space:]]*=[[:space:]]*\"[[:space:]]*(-Lsd|-n -Lf \/dev\/null)" | wc -l`
    
    # In case of snmptrapd.options line, .
    if [ $targetchk1 -ne 0 ]; then
        set $(echo $target)
        SNMPTRAPOPT=$2
        l1=1
    fi
    
    # In case of fi line, .
    if [ $l1 -eq 1 -a $targetchk2 -ne 0 ]; then
        l1=2
    fi
    
    # In case of OPTIONS line, .
    if [ $l1 -eq 2 -a $targetchk3 -ne 0 ]; then
        SNMPTRAPOPT=$SNMPTRAPRC
    fi
    
done < $SNMPTRAPRC

else
# RHEL7 or later
SNMPTRAPOPT="/etc/sysconfig/snmptrapd"

fi

#
# snmptrapd option setting for SVMCO
#
case $type in
	'install'|'upinstall')
		if [ ! -f $SNMPTRAPOPT ]; then
		    touch $SNMPTRAPOPT
		    chmod 755 $SNMPTRAPOPT
		    echo "OPTIONS=\"-n -Lf /dev/null -p /var/run/snmptrapd.pid\"" >> $SNMPTRAPOPT
		else
		    
		    cp $SNMPTRAPOPT $SNMPTRAPOPTBK
		    
		    ncnt=`cat $SNMPTRAPOPT | wc -l`
		    zcnt=`grep $ $SNMPTRAPOPT | wc -l`
		    
		    if [ $ncnt -ne $zcnt ]
			then
			echo "" >> $SNMPTRAPOPT
		    fi
		    
		    
		    l1=`egrep -c "^[[:space:]]*OPTIONS" $SNMPTRAPOPT`
		    l2=`egrep -c "^[[:space:]]*OPTIONS[[:space:]]*=[[:space:]]*\"[[:space:]]*.{OPTIONS}" $SNMPTRAPOPT`
		    
		    if [ $l1 -eq 0 ];then
		        echo "OPTIONS=\"-n -Lf /dev/null -p /var/run/snmptrapd.pid\"" >> $SNMPTRAPOPT
		    elif [ $l1 -eq $l2 ]; then
		        touch $SNMPTRAPOPT.org
		        chmod --reference $SNMPTRAPOPT $SNMPTRAPOPT.org
		        
		        echo "OPTIONS=\"-n -Lf /dev/null -p /var/run/snmptrapd.pid\"" >> $SNMPTRAPOPT.org
		        cat $SNMPTRAPOPT >> $SNMPTRAPOPT.org
		        mv $SNMPTRAPOPT.org $SNMPTRAPOPT
		        
		    else
		        trapd_opt_conv $SNMPTRAPOPT "^[[:space:]]*OPTIONS" "s/-Lsd/-n -Lf \/dev\/null/"
		    fi
		fi

		;;

	'uninstall')
		if [ -f $SNMPTRAPOPT ]; then
		    trapd_opt_conv $SNMPTRAPOPT "^[[:space:]]*OPTIONS" "s/-n -Lf \/dev\/null/-Lsd/"
		fi

		;;

esac

#---------------------------- snmptrapd definition start -----------------------------------

case $type in
	'install')
 
		if ! [ -f $SNMPTRPDFILE ]
		then
			echo "" >> $SNMPTRPDFILE
			echo "$STARTLINE"  >> $SNMPTRPDFILE
			NETSNMPVL1=`rpm -qa net-snmp | grep "net-snmp-[0-4]" | wc -l`
			if [ $NETSNMPVL1 -eq 0 ]
			then
				NETSNMPVL2=`rpm -qa net-snmp | grep "net-snmp-5\.[0-2]\." | wc -l`
				if [ $NETSNMPVL2 -eq 0 ]
				then
				        echo "snmpTrapdAddr udp:162,udp6:162" >> $SNMPTRPDFILE
      					echo "authCommunity execute $TRAPCOMMUNITY" >> $SNMPTRPDFILE
					echo "" >> $SNMPTRPDFILE
				fi
			fi
			for INPUTOID in $TRAPDOID
				do
					echo "traphandle $INPUTOID $TRAPCMD $TRAPOPT" >> $SNMPTRPDFILE
				done
			echo "$STOPLINE" >> $SNMPTRPDFILE
			echo "" >> $SNMPTRPDFILE
		else
			STARTCHK=`grep "$STARTLINE" $SNMPTRPDFILE | wc -w`
			if [ $STARTCHK -eq 0 ]
			then
				ncnt=`cat $SNMPTRPDFILE | wc -l`
				zcnt=`grep $ $SNMPTRPDFILE | wc -l`
				if [ $ncnt -ne $zcnt ]
					then
					echo "" >> $SNMPTRPDFILE
				fi
				echo "" >> $SNMPTRPDFILE
				echo "$STARTLINE"  >> $SNMPTRPDFILE
				NETSNMPVL1=`rpm -qa net-snmp | grep "net-snmp-[0-4]" | wc -l`
				if [ $NETSNMPVL1 -eq 0 ]
				then
					NETSNMPVL2=`rpm -qa net-snmp | grep "net-snmp-5\.[0-2]\." | wc -l`
					if [ $NETSNMPVL2 -eq 0 ]
					then
					        echo "snmpTrapdAddr udp:162,udp6:162" >> $SNMPTRPDFILE
						echo "authCommunity execute $TRAPCOMMUNITY" >> $SNMPTRPDFILE
						echo "" >> $SNMPTRPDFILE
					fi
				fi
				for INPUTOID in $TRAPDOID
					do
						echo "traphandle $INPUTOID $TRAPCMD $TRAPOPT" >> $SNMPTRPDFILE
					done
				echo "$STOPLINE" >> $SNMPTRPDFILE
				echo "" >> $SNMPTRPDFILE
			fi
		fi
		;;

    'upinstall')
                ## Delete SVMCO setting
                zcnt=`grep $ $SNMPTRPDFILE | wc -l`
                startl=`grep -n "$STARTLINE" $SNMPTRPDFILE | cut -d: -f1`
                stopl=`grep -n "$STOPLINE" $SNMPTRPDFILE | cut -d: -f1`

		if ! [ -z "$startl" -o -z "$stopl" ]
			then
			if [ $startl -lt $stopl ]
				then
				touch $TRAPCONFTMP
				chmod --reference $SNMPTRPDFILE $TRAPCONFTMP
				head -n `expr $startl - 1` $SNMPTRPDFILE > $TRAPCONFTMP
				tail -n `expr $zcnt - $stopl` $SNMPTRPDFILE >> $TRAPCONFTMP
				mv $TRAPCONFTMP $SNMPTRPDFILE

				if [ -e /etc/rsyslog.conf ]
                                    then
                                    restorecon -F $SNMPTRPDFILE
                                fi
			fi
		fi
                
                ## Add SVMCO setting
		if ! [ -f $SNMPTRPDFILE ]
		then
			echo "" >> $SNMPTRPDFILE
			echo "$STARTLINE"  >> $SNMPTRPDFILE
			NETSNMPVL1=`rpm -qa net-snmp | grep "net-snmp-[0-4]" | wc -l`
			if [ $NETSNMPVL1 -eq 0 ]
			then
				NETSNMPVL2=`rpm -qa net-snmp | grep "net-snmp-5\.[0-2]\." | wc -l`
				if [ $NETSNMPVL2 -eq 0 ]
				then
				        echo "snmpTrapdAddr udp:162,udp6:162" >> $SNMPTRPDFILE
					echo "authCommunity execute $TRAPCOMMUNITY" >> $SNMPTRPDFILE
					echo "" >> $SNMPTRPDFILE
				fi
			fi
			for INPUTOID in $TRAPDOID
				do
					echo "traphandle $INPUTOID $TRAPCMD $TRAPOPT" >> $SNMPTRPDFILE
				done
			echo "$STOPLINE" >> $SNMPTRPDFILE
			echo "" >> $SNMPTRPDFILE
		else
			STARTCHK=`grep "$STARTLINE" $SNMPTRPDFILE | wc -w`
			if [ $STARTCHK -eq 0 ]
			then
				ncnt=`cat $SNMPTRPDFILE | wc -l`
				zcnt=`grep $ $SNMPTRPDFILE | wc -l`
				if [ $ncnt -ne $zcnt ]
					then
					echo "" >> $SNMPTRPDFILE
				fi
				echo "" >> $SNMPTRPDFILE
				echo "$STARTLINE"  >> $SNMPTRPDFILE
				NETSNMPVL1=`rpm -qa net-snmp | grep "net-snmp-[0-4]" | wc -l`
				if [ $NETSNMPVL1 -eq 0 ]
				then
					NETSNMPVL2=`rpm -qa net-snmp | grep "net-snmp-5\.[0-2]\." | wc -l`
					if [ $NETSNMPVL2 -eq 0 ]
					then
	    				        echo "snmpTrapdAddr udp:162,udp6:162" >> $SNMPTRPDFILE
						echo "authCommunity execute $TRAPCOMMUNITY" >> $SNMPTRPDFILE
						echo "" >> $SNMPTRPDFILE
					fi
				fi
				for INPUTOID in $TRAPDOID
					do
						echo "traphandle $INPUTOID $TRAPCMD $TRAPOPT" >> $SNMPTRPDFILE
					done
				echo "$STOPLINE" >> $SNMPTRPDFILE
				echo "" >> $SNMPTRPDFILE
			fi
		fi
		;;
		
	'uninstall')

		zcnt=`grep $ $SNMPTRPDFILE | wc -l`
		startl=`grep -n "$STARTLINE" $SNMPTRPDFILE | cut -d: -f1`
		stopl=`grep -n "$STOPLINE" $SNMPTRPDFILE | cut -d: -f1`

		if ! [ -z "$startl" -o -z "$stopl" ]
			then
			if [ $startl -lt $stopl ]
				then
				touch $TRAPCONFTMP
				chmod --reference $SNMPTRPDFILE $TRAPCONFTMP
				head -n `expr $startl - 1` $SNMPTRPDFILE > $TRAPCONFTMP
				tail -n `expr $zcnt - $stopl` $SNMPTRPDFILE >> $TRAPCONFTMP
				mv $TRAPCONFTMP $SNMPTRPDFILE

				if [ -e /etc/rsyslog.conf ]
                                    then
                                    restorecon -F $SNMPTRPDFILE
                                fi
			fi
		fi
		;;

esac


#------------------------------ snmpd/snmptrapd start  ---------------------------------

case $type in
	'install')
		
		if [ $level1 -ne 1 ]; then
		    if [ $OS_TYPE -ne 1 ]; then
			/sbin/service snmptrapd start  > /dev/null 2>&1
		    else
			/bin/systemctl start snmptrapd > /dev/null 2>&1
		    fi
		fi
		
		if [ $OS_TYPE -ne 1 ]; then
		    chkconfig --add snmptrapd
		    chkconfig snmptrapd on
		else
		    /bin/systemctl enable snmptrapd > /dev/null 2>&1
		fi

		;;

	'upinstall')
		if [ $level1 -ne 1 ]; then
		    if [ $OS_TYPE -ne 1 ]; then
			/sbin/service snmptrapd start  > /dev/null 2>&1
		    else
			/bin/systemctl start snmptrapd > /dev/null 2>&1
		    fi
		fi

		;;

esac

exit 0
